/*********************************************************************************************
	In-Memory Wall of Shame
	The following statements will fail.  These are options not currently available
	to In-Memory OLTP tables.
*********************************************************************************************/

--Fail 1:  Unique constraints
CREATE TABLE PoliceData.InMemoryFail_UniqueConstraints
(
	Id INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 100),
	IncidentNumber VARCHAR(19) NOT NULL UNIQUE
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

GO

--Fail 2:  Foreign Key constraints
CREATE TABLE PoliceData.InMemoryFail_FKConstraints
(
	Id INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 100),
	IncidentNumber VARCHAR(19) NOT NULL FOREIGN KEY REFERENCES PoliceData.Incident(IncidentNumber)
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

GO

--Fail 3:  "Advanced" data types like Geography
CREATE TABLE PoliceData.InMemoryFail_Geography
(
	Id INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 100),
	IncidentNumber GEOGRAPHY
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

GO

--Fail 4:  CREATE INDEX
CREATE TABLE PoliceData.InMemoryFail_CreateIndex
(
	Id INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 100),
	IncidentNumber VARCHAR(10) NOT NULL
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

CREATE INDEX [IX_IndexFail] ON PoliceData.InMemoryFail_CreateIndex
(
	IncidentNumber
);

DROP TABLE PoliceData.InMemoryFail_CreateIndex;

GO

--Fail 5:  Indexing NULLable Columns
CREATE TABLE PoliceData.InMemoryFail_NULLableIndex
(
	IncidentID INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 500000),
	IncidentNumber VARCHAR(19) COLLATE Latin1_General_100_BIN2 NULL

	INDEX [IX_Fail_IncidentNumber] NONCLUSTERED(IncidentNumber)
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

GO

--Fail 6:  Non-Binary Collations on (N)VARCHAR Indexes
CREATE TABLE PoliceData.InMemoryFail_NULLableIndex
(
	IncidentID INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 500000),
	IncidentNumber VARCHAR(19) COLLATE SQL_Latin1_General_CP1_CI_AS NOT NULL

	INDEX [IX_Fail_IncidentNumber] NONCLUSTERED(IncidentNumber)
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

GO

--Fail 7:  subqueries in a natively compiled stored procedure
CREATE PROCEDURE dbo.IncidentInMemory_GetIncidentInMemoryTest
(
@BeatID INT
)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	SELECT
		IncidentID,
		IncidentNumber,
		LocalCrimeReportingCode,
		IncidentDate,
		Beat,
		LocationID
	FROM
		PoliceData.IncidentInMemory iim
		OUTER APPLY
		(
			SELECT 1 AS val
		) v
	WHERE
		Beat = @BeatID;

END
GO

--Fail 8:  COALESCE or NULLIF in a natively compiled stored procedure
CREATE PROCEDURE dbo.IncidentInMemory_GetIncidentInMemoryTest
(
@BeatID INT
)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	SELECT
		IncidentID,
		IncidentNumber,
		LocalCrimeReportingCode,
		IncidentDate,
		Beat,
		COALESCE(LocationID, 3) AS LocationID
	FROM
		PoliceData.IncidentInMemory iim
	WHERE
		Beat = @BeatID;

END
GO

--Fail 9:  Temp tables in a natively compiled stored procedure
CREATE PROCEDURE dbo.IncidentInMemory_GetIncidentInMemoryTest
(
@BeatID INT
)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	CREATE TABLE #Test (Id int);

	SELECT
		IncidentID,
		IncidentNumber,
		LocalCrimeReportingCode,
		IncidentDate,
		Beat,
		LocationID
	FROM
		PoliceData.IncidentInMemory iim
	WHERE
		Beat = @BeatID;

END
GO

--Fail 10:  Access on-disk tables
CREATE PROCEDURE dbo.IncidentInMemory_GetIncidentInMemoryTest
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	SELECT
		N
	FROM
		dbo.Tally;

END
GO

--Quasi-Fail 11:  SELECT *
--Quasi-fail for two reasons:
	--1)  This is a SCHEMABINDING rule and not In-Memory OLTP
	--2)  You shouldn't be using SELECT *!
CREATE PROCEDURE dbo.IncidentInMemory_GetIncidentInMemoryTest
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	SELECT
		*
	FROM
		PoliceData.IncidentInMemory;

END
GO

/*********************************************************************************************
	End In-Memory Wall of Shame
*********************************************************************************************/


/*********************************************************************************************
	In-Memory Info Wall of Shame
	The following statements will NOT fail even though plenty of info is available saying so.
	This shows that stuff changes between CTP & Release.
*********************************************************************************************/

--Identity columns ARE supported!  Most info will say they aren't.
CREATE TABLE PoliceData.InMemoryInfoFail_IdentityInteger
(
	Id INT IDENTITY(1,1) NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 500000),
	IncidentNumber VARCHAR(19)
)
WITH
(
	MEMORY_OPTIMIZED = ON,
	DURABILITY = SCHEMA_ONLY
);

INSERT INTO PoliceData.InMemoryInfoFail_IdentityInteger(IncidentNumber) VALUES ('This can''t work!');
SELECT * FROM PoliceData.InMemoryInfoFail_IdentityInteger;

GO

CREATE PROCEDURE dbo.InMemoryInfoFail_IdentityIntegerTest
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS
BEGIN ATOMIC
WITH (TRANSACTION ISOLATION LEVEL=SNAPSHOT, LANGUAGE=N'us_english')

	INSERT INTO PoliceData.InMemoryInfoFail_IdentityInteger(IncidentNumber) VALUES ('This can''t work!');

END
GO

EXEC dbo.InMemoryInfoFail_IdentityIntegerTest;

GO

DROP PROCEDURE dbo.InMemoryInfoFail_IdentityIntegerTest;
DROP TABLE PoliceData.InMemoryInfoFail_IdentityInteger;


/*********************************************************************************************
	End In-Memory Info Wall of Shame
*********************************************************************************************/